"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.default = createCompiler;

var _webpack = _interopRequireDefault(require("webpack"));

var _lodash = _interopRequireDefault(require("lodash"));

var _chalk = _interopRequireDefault(require("chalk"));

var _forkTsCheckerWebpackPlugin = _interopRequireDefault(require("fork-ts-checker-webpack-plugin"));

var _printCompileMessages = _interopRequireDefault(require("./printCompileMessages"));

var _typescriptFormatter = _interopRequireDefault(require("./typescriptFormatter"));

var _webpack2 = _interopRequireDefault(require("../webpack/webpack.config"));

var _logger = _interopRequireDefault(require("./logger"));

function createCompiler(options) {
  const config = (0, _webpack2.default)(options);
  const useTypeScript = options.useTypeScript;
  (0, _logger.default)("Starting compile...\n");
  let compiler;

  try {
    compiler = (0, _webpack.default)(config);
  } catch (err) {
    console.log(_chalk.default.red("Failed to compile."));
    console.log();
    console.log(err.message || err);
    console.log();
    process.exit(1);
  }

  let isFirstCompile = true;
  let tsMessagesPromise;
  let tsMessagesResolver;

  if (useTypeScript) {
    compiler.hooks.beforeCompile.tap("beforeCompile", () => {
      if (!isFirstCompile) {
        (0, _logger.default)("Compiling...\n");
      }

      tsMessagesPromise = new Promise(resolve => {
        tsMessagesResolver = msgs => resolve(msgs);
      });
    });

    _forkTsCheckerWebpackPlugin.default.getCompilerHooks(compiler).receive.tap("afterTypeScriptCheck", (diagnostics, lints) => {
      const allMsgs = [...diagnostics, ...lints];

      const format = message => `${message.file}\n${(0, _typescriptFormatter.default)(message, true)}`;

      tsMessagesResolver({
        errors: allMsgs.filter(msg => msg.severity === "error").map(format),
        warnings: allMsgs.filter(msg => msg.severity === "warning").map(format)
      });
    });
  }

  compiler.hooks.done.tap("done", async stats => {
    const statsData = stats.toJson({
      all: false,
      warnings: true,
      errors: true
    });

    if (useTypeScript && statsData.errors.length === 0) {
      const delayedMsg = setTimeout(() => {
        console.log(_chalk.default.yellow("Files successfully emitted, waiting for typecheck results...\n"));
      }, 100);
      const messages = await tsMessagesPromise;
      clearTimeout(delayedMsg);
      stats.compilation.errors.push(...messages.errors);
      stats.compilation.warnings.push(...messages.warnings);
    }

    isFirstCompile = false;
    (0, _printCompileMessages.default)(stats, config);
  });
  return compiler;
}